import { match } from 'path-to-regexp';
const DEFAULT_RESPONSE_HEADERS = {
    'Content-Type': 'application/json; charset=utf-8',
    'Access-Control-Allow-Origin': '*',
    'Access-Control-Allow-Methods': '*',
    'Access-Control-Allow-Headers': '*',
};
export class RestApi {
    routes = [];
    addRouteHandler(route, requestHandler) {
        this.routes = [
            ...this.routes,
            {
                route,
                requestHandler,
            },
        ];
        return this;
    }
    async handleRequest(proxyEvent) {
        try {
            const url = new URL(`${proxyEvent.headers['origin'] || 'https://no-origin'}${proxyEvent.path}`);
            const routeHandler = this.getMatchingRouteHandler(url, proxyEvent.httpMethod);
            if (!routeHandler && proxyEvent.httpMethod === 'OPTIONS') {
                const allowedHttpMethods = this.getMatchingRouteHandlers(url).map(({ routeHandler }) => routeHandler.route.method);
                return {
                    statusCode: 200,
                    body: '',
                    headers: {
                        ...DEFAULT_RESPONSE_HEADERS,
                        'Access-Control-Allow-Methods': allowedHttpMethods.join(','),
                    },
                };
            }
            if (!routeHandler)
                return {
                    statusCode: 405,
                    headers: DEFAULT_RESPONSE_HEADERS,
                    body: `Method ${proxyEvent.httpMethod} not allowed on ${proxyEvent.path}`,
                };
            const { routeHandler: { requestHandler, route }, pathParams, queryParams } = routeHandler;
            const userId = proxyEvent.requestContext.authorizer?.principalId;
            if (route.requireAuthentication && !userId)
                return { statusCode: 401, body: '' };
            const responseBody = await requestHandler({
                pathParams,
                queryParams,
                headers: proxyEvent.headers,
                body: proxyEvent.body && JSON.parse(proxyEvent.body),
                userId,
            });
            return {
                statusCode: responseBody.status,
                body: responseBody.body ? JSON.stringify(responseBody.body) : '',
                headers: {
                    ...DEFAULT_RESPONSE_HEADERS,
                    ...(responseBody.headers || {}),
                },
            };
        }
        catch (error) {
            console.error(error);
            return {
                statusCode: 500,
                headers: DEFAULT_RESPONSE_HEADERS,
                body: JSON.stringify({ error }),
            };
        }
    }
    getMatchingRouteHandlers(url) {
        return this.routes.reduce((matchingRouteHandlers, routeHandler) => {
            const pathParams = this.extractPathParams(routeHandler, url);
            const queryParams = this.extractQueryParams(url);
            const matchingRouteHandler = {
                routeHandler,
                pathParams,
                queryParams,
            };
            return pathParams
                ? [...matchingRouteHandlers, matchingRouteHandler]
                : matchingRouteHandlers;
        }, []);
    }
    getMatchingRouteHandler(url, httpMethod) {
        return this.getMatchingRouteHandlers(url)
            .find(({ routeHandler }) => routeHandler.route.method === httpMethod);
    }
    extractPathParams(routeHandler, url) {
        const matcher = match(routeHandler.route.path, { decode: decodeURIComponent });
        return (matcher(url.pathname) || null)?.params;
    }
    extractQueryParams(url) {
        const queryParams = {};
        for (const [key, value] of url.searchParams.entries()) {
            if (key in queryParams) {
                const originalValue = queryParams[key];
                queryParams[key] = Array.isArray(originalValue)
                    ? [...originalValue, value]
                    : [originalValue, value];
            }
            else {
                queryParams[key] = value;
            }
        }
        return queryParams;
    }
}
//# sourceMappingURL=RestApi.js.map